/**
 * shutdown.C: This file contains functionality which implements the OperServ SHUTDOWN command.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005-2006 Atheme Development Group
 * Rights to this code are documented in doc/pod/license.pod.
 *
 * $Id: shutdown.C,v 1.7 2007/09/22 14:27:28 pippijn Exp $
 */

#include "atheme.h"
#include <ermyth/module.h>
#include <ermyth/database.h>

static char const rcsid[] = "$Id: shutdown.C,v 1.7 2007/09/22 14:27:28 pippijn Exp $";

REGISTER_MODULE ("operserv/shutdown", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void os_cmd_shutdown (sourceinfo_t *si, int parc, char *parv[]);

command_t const os_shutdown = { "SHUTDOWN", N_("Shuts down services."), PRIV_ADMIN, 0, os_cmd_shutdown };

E cmdvec os_cmdtree;
E helpvec os_helptree;

bool
_modinit (module *m)
{
  os_cmdtree << os_shutdown;
  help_addentry (os_helptree, "SHUTDOWN", "help/operserv/shutdown", NULL);

  return true;
}

void
_moddeinit ()
{
  os_cmdtree >> os_shutdown;
  help_delentry (os_helptree, "SHUTDOWN");
}

static void
os_cmd_shutdown (sourceinfo_t *si, int parc, char *parv[])
{
  snoop ("UPDATE: \2%s\2", get_oper_name (si));
  wallops ("Updating database by request of \2%s\2.", get_oper_name (si));
  expire_check (NULL);
  backend->save ();

  logcommand (si, CMDLOG_ADMIN, "SHUTDOWN");
  snoop ("SHUTDOWN: \2%s\2", get_oper_name (si));
  wallops ("Shutting down by request of \2%s\2.", get_oper_name (si));

  runflags |= RF_SHUTDOWN;
}

/* vim:cinoptions=>s,e0,n0,f0,{0,}0,^0,=s,ps,t0,c3,+s,(2s,us,)20,*30,gs,hs
 * vim:ts=8
 * vim:sw=8
 * vim:noexpandtab
 */
